unit zipdllImport;

{
Define o carregamento automtico das funes da DLL. Se estiver definido, assim
que essa unit  inicializada (durante a execuo de um programa) a dll 
carregada para memria (o que significa que ela deve estar presente no sistema
do usurio no momento em que o programa for rodado.
}
{.$DEFINE ZIPDLLAUTOLOAD}

interface

uses
     windows, classes, graphics, jpeg, sysUtils, extCtrls, controls;

const
     ZIPDLL = 'zipdll.dll';

type
     PLongWord = ^LongWord;
     TZipDLLCallBackProc = procedure (nome: PChar; tam: PLongWord; arq: Pointer);

procedure zipInitDll;
procedure zipFinalizeDLL;
function zipLerComoStrings(arquivo: string): TStrings;
function zipLerComoBitmap(arquivo: string): TBitmap;
function zipLerComoJpeg(arquivo: string): TJpegImage;
procedure zipCarregarArq(nomeArq: string); overload;
function zipLerArquivo(nomeArq: string; var tam: LongWord): pointer; overload;
procedure zipLerParaTImage(img: TImage; arq: string);
function zipAcharTImage(nome: string; control: TWinControl): TImage;
procedure zipLerParaStream(arquivo: string; str: TStream);

type
     TZipCarregarArq = procedure (nomeArq: PChar);
     TZipListarArqs = function: PChar;
     TZipLerArqAtual = function (tam: PLongWord): pointer;
     TZipLerArquivo = function (nomeArq: PChar; tam: PLongWord): pointer;
     TZipLerNomeArqContem = function (nomearq: PChar; tam: PLongWord): pointer;
     TZipFecharArq = procedure;
     TZipLimparMem = procedure;
     TZipLerTodoArqContem = procedure (nomearq: PChar; proc: pointer);
     TZipArquivoExiste = function (nomearq: PChar; proc: pointer): boolean;

var
     zipdllCarregarArq: TZipCarregarArq;
     zipdllListarArqs: TZipListarArqs;
     zipdllLerArqAtual: TZipLerArqAtual;
     zipdllLerArquivo: TZipLerArquivo;
     zipdllLerNomeArqContem: TZipLerNomeArqContem;
     zipdllFecharArq: TZipFecharArq;
     zipdllLimparMem: TZipLimparMem;
     zipdllLerTodoArqContem: TZipLerTodoArqContem;
     zipdllArquivoExiste: TZipArquivoExiste;

implementation

var
     dllhandle: THandle;

procedure zipInitDll;
begin
     if dllhandle > 0 then exit;

     dllhandle:= LoadLibrary(PChar(ZIPDLL));
     if dllhandle = 0 then begin
          raise Exception.Create('Falha na inicializao da biblioteca zipdll.dll');
          exit;
     end;

     @zipdllCarregarArq:= GetProcAddress(dllhandle, 'zipCarregarArq');
     @zipdllListarArqs:= GetProcAddress(dllhandle, 'zipListarArqs');
     @zipdllLerArqAtual:= GetProcAddress(dllhandle, 'zipLerArqAtual');
     @zipdllLerArquivo:= GetProcAddress(dllhandle, 'zipLerArquivo');
     @zipdllLerNomeArqContem:= GetProcAddress(dllhandle, 'zipLerNomeArqContem');
     @zipdllFecharArq:= GetProcAddress(dllhandle, 'zipFecharArq');
     @zipdllLimparMem:= GetProcAddress(dllhandle, 'zipLimparMem');
     @zipdllLerTodoArqContem:= GetProcAddress(dllhandle, 'zipLerTodoArqContem');
     @zipdllArquivoExiste:= GetProcAddress(dllhandle, 'zipArquivoExiste');
end;

procedure zipFinalizeDLL;
begin
     if dllhandle < 1 then exit;

     if not FreeLibrary(dllhandle) then
          raise Exception.Create('Erro ao tentar finalizar biblioteca zipdll.dll');
     dllhandle:= 0;

     @zipdllCarregarArq:= nil;
     @zipdllListarArqs:= nil;
     @zipdllLerArqAtual:= nil;
     @zipdllLerArquivo:= nil;
     @zipdllLerNomeArqContem:= nil;
     @zipdllFecharArq:= nil;
     @zipdllLimparMem:= nil;
     @zipdllLerTodoArqContem:= nil;
     @zipdllArquivoExiste:= nil;
end;

function zipLerComoStrings(arquivo: string): TStrings;
var
arq: pointer;
tam: longword;
aux: string;
begin
     result:= TStringList.create;
     arq:= zipdllLerArquivo(PChar(arquivo), @tam);
     if assigned(arq) then begin
          SetLength(aux, tam);
          StrLCopy(PChar(aux), arq, tam);
          result.SetText(PChar(aux));
          aux:= '';
     end;
end;

function zipLerComoBitmap(arquivo: string): TBitmap;
var
arq: pointer;
tam: longword;
str: TMemoryStream;
begin
     result:= TBitmap.create;
     arq:= zipdllLerArquivo(PChar(arquivo), @tam);
     if assigned(arq) then begin
          str:= TMemoryStream.create;
          str.write(arq^, tam);
          str.position:= 0;
          result.LoadFromStream(str);
          str.free;
     end;
end;

function zipLerComoJpeg(arquivo: string): TJPEGImage;
var
arq: pointer;
tam: longword;
str: TMemoryStream;
begin
     result:= TJPEGImage.create;
     arq:= zipdllLerArquivo(PChar(arquivo), @tam);
     if assigned(arq) then begin
          str:= TMemoryStream.create;
          str.write(arq^, tam);
          str.position:= 0;
          result.LoadFromStream(str);
          str.free;
     end else begin
          result.Assign(TBitmap.create);
     end;
end;

procedure zipCarregarArq(nomeArq: string);
begin
     zipdllCarregarArq(Pchar(nomearq));
end;

function zipLerArquivo(nomeArq: string; var tam: LongWord): pointer;
begin
     result:= zipdllLerArquivo(PChar(nomeArq), @tam);
end;

procedure zipLerParaTImage(img: TImage; arq: string);
var
graf: TGraphic;
begin
     //dependendo do tipo, carregar apropriadamente
     if ExtractFileExt(arq) = '.bmp' then begin
          graf:= zipLerComoBitmap(arq);
          //TBitmap(graf).PixelFormat:= pf32bit;
     end else
          graf:= zipLerComoJpeg(arq);

     //colocar na TImage
     img.picture.assign(graf);

     //liberar memria
     graf.free;
end;

function zipAcharTImage(nome: string; control: TWinControl): TImage;
var
i: integer;
begin
     result:= nil;
     for i:= 0 to control.controlCount -1 do
          if control.Controls[i].Name = nome then begin
               if not control.Controls[i].InheritsFrom(TImage) then
                    raise Exception.create('Controle com nome ' + nome + ' no  uma TImage');
               result:= TImage(control.controls[i]);
               break;
          end;

     if not assigned(result) then
          for i:= 0 to control.ControlCount -1 do
               if control.Controls[i].InheritsFrom(TWinControl) then begin
                    result:= zipAcharTImage(nome, TWinControl(control.controls[i]));
                    if assigned(result) then break;
               end;

     if not assigned(result) then
          raise Exception.Create('Controle ' + nome + ' no encontrado');
end;

procedure zipLerParaStream(arquivo: string; str: TStream);
var
arq: pointer;
tam: longword;
begin
     arq:= zipdllLerArquivo(PChar(arquivo), @tam);
     if assigned(arq) then begin
          str.write(arq^, tam);
          str.position:= 0;
     end;
end;

initialization
     dllhandle:= 0;
     {$IFDEF ZIPDLLAUTOLOAD}
     zipInitDll;
     {$ENDIF}

finalization
     zipFinalizeDLL;

end.
